﻿using System;
using System.Windows.Forms;
using AddinExpress.MSO;
using Outlook = Microsoft.Office.Interop.Outlook;
using System.Runtime.InteropServices;

namespace adx_ol_cs_ProcessItemEvents
{
    /// <summary>
    /// Add-in Express Outlook Item Events Class
    /// </summary>
    public class OutlookItemEventsClass : AddinExpress.MSO.ADXOutlookItemEvents
    {
        public OutlookItemEventsClass(AddinExpress.MSO.ADXAddinModule module)
            : base(module)
        {
        }

        /// <summary>
        /// The Process event is raised when the user opens an Outlook item in an Inspector. 
        /// </summary>
        /// <param name="e">Set the Cancel property to true in order to cancel the open event.</param>
        public override void ProcessOpen(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mailOpen = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "You've read a mail with Subject: " + mailOpen.Subject + " today";
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null) Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        /// <summary>
        /// The Read event is similar to the Open event except for the fact that it is 
        /// also raised when the user selects the item in a view that supports in-cell editing.
        /// </summary>
        public override void ProcessRead()
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "You've read a mail with Subject: " + mail.Subject + " today";
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null) Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        /// <summary>
        /// When the user replies on an Outlook Item, the Reply event is raised.
        /// </summary>
        /// <param name="response">A reference to the new item that is being sent in response</param>
        /// <param name="e">Set the Cancel property to true in order to cancel the reply event.</param>
        public override void ProcessReply(object response, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (response is Outlook._MailItem)
            {
                Outlook._MailItem mailResponse = (Outlook._MailItem)response;
                mailResponse.BCC = "someone@somewhere.com";
            }
        }

        /// <summary>
        /// The ProcessReplyAll is similar to the ProcessReply method. This event is raised when the user replies to all, on an Outlook item.
        /// </summary>
        /// <param name="response">A reference to the new item that is being sent in response</param>
        /// <param name="e">Set the Cancel property to true in order to cancel the reply all event.</param>
        public override void ProcessReplyAll(object response, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (response is Outlook._MailItem)
            {
                Outlook._MailItem mailResponse = (Outlook._MailItem)response;
                mailResponse.BCC = "everyone@everywhere.com";
            }
        }

        /// <summary>
        /// When the user clicks send or you invoke the Send method on an Outlook Item, this event will fire
        /// </summary>
        /// <param name="e">Set the Cancel property to true in order to cancel the send event.</param>
        public override void ProcessSend(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "Sent an e-mail to " + mail.To;
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null)
                        Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        public override void ProcessAttachmentAdd(object attachment)
        {
            // TODO: Add some code
        }

        public override void ProcessAttachmentRead(object attachment)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentSave(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeCheckNames(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessClose(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessCustomAction(object action, object response, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessCustomPropertyChange(string name)
        {
            // TODO: Add some code
        }

        public override void ProcessForward(object forward, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessPropertyChange(string name)
        {
            // TODO: Add some code
        }

        public override void ProcessWrite(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeDelete(object item, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessAttachmentRemove(object attachment)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentAdd(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentPreview(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentRead(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentWriteToTempFile(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessUnload()
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAutoSave(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeRead()
        {
            // TODO: Add some code
        }

        public override void ProcessAfterWrite()
        {
            // TODO: Add some code
        }
    }
}

